﻿;----------------------------------------------------
;   Developed by: Kim Strandli
;   www.kimstrandli.com
;
;   Thanks to "Denton" for initial Wacom-script.
;   https://autohotkey.com/board/topic/46687-wacom-tablet-and-switching-between-2-monitors/?p=437622
;
;-------------------------------------------------
#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
#SingleInstance, Force
#persistent
SendMode Input  ; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
CoordMode, Mouse, Screen
;Wacom Display Toggle


version := "1.1.2"
released := "2021/07"

Menu, Tray, NoStandard
Menu, Tray, Add , Suspend, Sus
Menu, Tray, Add , Restart, Rel
Menu, Tray, Add , Config, OpenConfigGui
Menu, Tray, Add , About, About
Menu, Tray, Add , Exit, Exi
Menu, Tray, Default, Exit


iniSettingFile := "KsWacomDisplayToggle.ini"

settingsDictionary := []
TempHotkeys := []
HotkeysCurrent := []
DisplaySettings := []
DeviceMapping := []
Cintiq_PenName := "Grip Pen"
Intuos_PenName := "Grip Pen"


GetDeviceMapping() {
   global DeviceMapping
   for index, deviceType in ["intuos", "cintiq"] {
      DeviceMapping[deviceType] := ""
   
      for index, element in ["Button3", "Button4"] {
         ControlGetText, buttonText, %element%, Wacom Tablet Properties
         
         if InStr(buttonText, "Pen", CaseSensitive:=false) {
            DeviceMapping["defaultPenName"] := element
         }
         
         if InStr(buttonText, deviceType, CaseSensitive:=false) {
            DeviceMapping[deviceType] := element
            ;MsgBox "Mapped %deviceType%-device to %buttonText% - %element%"
         }
      }
   }
}




LoadSettings(){
   global DisplaySettings
   global iniSettingFile
   global settingsDictionary

   if !FileExist(iniSettingFile)
      makeDefaultSettings() 

   IniRead, SectionNames, %iniSettingFile%
   for each, Section in StrSplit(SectionNames, "`n") {
      IniRead, OutputVar_Section, %iniSettingFile%, %Section%
         
      for index, line in StrSplit(OutputVar_Section, "`n") ; Read each line in the ini-file
      
         if (InStr(Section, "monitor")) {
            RegExMatch(line, "(.*?)=(.*)", $)
            , DisplaySettings[Section, $1] := $2           
         }

         else {
            RegExMatch(line, "(.*?)=(.*)", $)
            , settingsDictionary[Section, $1] := $2
         }
      }
   return
}


makeDefaultSettings(){
   global iniSettingFile

   MsgBox, 1, KS_WacomDisplayToggle, No settings found!`nMaking default settings in %iniSettingFile% next to the script.`n`nOpen the config to assign hotkeys.

   IniWrite, 1, %iniSettingFile%, settings, DisableVerticalMovement

   IniWrite, 2, %iniSettingFile%, monitor1, force
   IniWrite, 1, %iniSettingFile%, monitor2, force

   return
}



GetActiveWacomMonitor_intuos() {
   ControlGet, monitor, choice,,ComboBox2,Wacom Tablet Properties
   StringLower, monitor, monitor
   monitorID := StrReplace(monitor, A_Space, "")
   return monitorID
}


GetActiveWacomMonitor_cintiq() {
   ControlGet, monitor, choice,,ComboBox1,Wacom Tablet Properties
   StringLower, monitor, monitor
   monitorID := substr(monitor, (strlen(monitor) -0))
   monitorID = monitor%monitorID%
   return monitorID
}


GetDefaultPen() {
   WinActivate,Wacom Tablet Properties
   WinWait,Wacom Tablet Properties
   
   for index, element in ["Button3", "Button4", "Button5", "Button6", "Button7", "Button8", "Button9", "Button10"] {
      ControlGetText, buttonText, %element%, Wacom Tablet Properties
      if InStr(buttonText, "Pen", CaseSensitive:=false) {
         ControlGetText, penName, %element%, Wacom Tablet Properties
         return penName
         }
      }
}

GetDefaultDevice() {
    IfWinNotExist Wacom Tablet Properties
    {
       ;If control panel is not open, open it and set to the third tab for active device (Mapping for intuos or calibration for cintiq), then minimize
       run,"C:\Program Files\Tablet\Wacom\Professional_CPL.exe" "/language EN"
       WinWait,Wacom Tablet Properties       
    }
    
   ControlGetText, buttonText, Button3, Wacom Tablet Properties
   if InStr(buttonText, "Intuos", CaseSensitive:=false) {
      ;MsgBox, Getting default device - %buttonText% - Found Intuos
      return "intuos"
   }
   if InStr(buttonText, "Cintiq", CaseSensitive:=false) {
      ;MsgBox, Getting default device - %buttonText% - Found Cintiq
      return "cintiq"
   }
}

ClickDeviceGUIButtons(device) {
   global settingsDictionary
   global DeviceMapping
   global Cintiq_PenName
   global Intuos_PenName
   
   WinActivate,Wacom Tablet Properties
   WinWait,Wacom Tablet Properties
   
   deviceButton := DeviceMapping[device]
   ControlClick,%deviceButton%,Wacom Tablet Properties
   WinWait,Wacom Tablet Properties
   
   if (device = "cintiq") {
      penName := settingsDictionary["settings", "CintiqPenName"]
   }
   if (device = "intuos") {
      penName := settingsDictionary["settings", "IntuosPenName"]
   }
   if (penName = "") {
      penName := GetDefaultPen()
      ;MsgBox, % "Got default pen: " . penName
   }
   
   if penName {
      ControlClick,%penName%,Wacom Tablet Properties
   }
   
   ControlClick,x140 y220,Wacom Tablet Properties ;third tab location
   WinMinimize,Wacom Tablet Properties
   Return
}

WacomPreCheck(device) {
   global DeviceMapping
   global Cintiq_PenName
      
   IfWinNotExist Wacom Tablet Properties
   {
      ;If control panel is not open, open it and set to the third tab for active device (Mapping for intuos or calibration for cintiq), then minimize
      run, "C:\Program Files\Tablet\Wacom\Professional_CPL.exe" "/language EN"
      WinWait,Wacom Tablet Properties
      GetDeviceMapping()
      ClickDeviceGUIButtons(device)
   }
    
   GetDeviceMapping()
   deviceButton := DeviceMapping[device]
   if (deviceButton = "") {
      MsgBox, Did not find mapped device for %device%
      return False
   }
   
   
    ;Still check if window exists
    ifWinExist Wacom Tablet Properties
    {
      ;Check if Control Panel has the right tab open.
      if (device = "intuos") {
         ControlGet,mapmode,FindString,Full,ComboBox2,Wacom Tablet Properties
          ifNotEqual,mapmode,1
          {
             ClickDeviceGUIButtons(device)
         }
         return true
      }
      
      if (device = "cintiq") {
        ControlGet, button, Visible,, Calibrate..., Wacom Tablet Properties
         if !(button) {
            ClickDeviceGUIButtons(device)
         }
         return true
      }
   }
   else {
      return false
   }
}



GetMonitorMouse() {
	MouseGetPos, x, y
	SysGet, MonCount, MonitorCount
	Loop, %MonCount%
	{
		SysGet, Mon, Monitor, %A_Index%
		if x between %MonLeft% and %MonRight%
		{
			if y between %MonTop% and %MonBottom% 
			{
				return A_Index
			}
		}
	}
}

GetMouseDirection() {
   MouseGetPos, xposOld, yposOld
   Sleep, 5
   MouseGetPos, xpos, ypos

   if settingsDictionary["settings"]["disableVerticalTrack"] {
      xposLength := 10
      yposLength := 0
   }
   else {
      xposLength := Abs(xpos - xposOld)
      yposLength := Abs(ypos - yposOld)
   }
   
    if (xposLength > yposLength) {
        if (xposOld < xpos) {
            return "right"
        }
        else if (xposOld > xpos) {
            return "left"
        }
    }
    else {
        if (yposOld < ypos){
            return "down"
        }
        else if (yposOld > ypos) {
            return "up"
        }
    }
    return
}

ActiveMouseBtnCheck() {
   ;Check for any active mouse buttons. If so, keep held and switch to other monitor.
   GetKeyState, state, LButton, P
   if state = U
   {
     GetKeyState, state, RButton, P
     if state = U
     {
       GetKeyState, state,	MButton, P
       if state = U
       {
         WinActivate,Program Manager
         return
       }
     }
   }
   else {
       return
   }
}

ToggleFullCoverage() {
   if not WacomPreCheck(intuos) {
      return
   }
   MonitorID := GetActiveWacomMonitor_intuos()
   
   ; Check if current "screen area" is set to a specific monitor.
   if (InStr(MonitorID, "monitor"))
   {
      Control, Choose,1,ComboBox2,Wacom Tablet Properties
   }
   else ; We assume it's set to "full"
   {
      cursorMonitor := GetMonitorMouse()
      if cursorMonitor {
         comboBoxTarget := (cursorMonitor + 2)
         Control, Choose,%comboBoxTarget%,ComboBox2,Wacom Tablet Properties
      }
      else {
         Control, Choose,3,ComboBox2,Wacom Tablet Properties
      }
   }

   ActiveMouseBtnCheck()
   return
}

ChangeTargetMonitor(device) {
   global DisplaySettings
   
   if (device = "default") {
      device := GetDefaultDevice()
   }
   
   if not WacomPreCheck(device) {
      return
   }
      
   if (device = "intuos") {
      MonitorID := GetActiveWacomMonitor_intuos()
   }
   if (device = "cintiq") {
      MonitorID := GetActiveWacomMonitor_cintiq()
   }
   
   forceTarget := DisplaySettings[MonitorID]["force"]
   if forceTarget {
      MonitorTarget := forceTarget
   }
   else {
      MouseGesture := GetMouseDirection()
      if not MouseGesture {
         return
      }
      MonitorTarget := % DisplaySettings[MonitorID, MouseGesture]
   }
   
   if not MonitorTarget {
      return
   }

   if (device = "intuos") {
      comboBoxTarget := (MonitorTarget + 2)
      Control, Choose,%comboBoxTarget%,ComboBox2,Wacom Tablet Properties
   }
   if (device = "cintiq") {
      Control, Choose,%MonitorTarget%,ComboBox1,Wacom Tablet Properties
   }


   ActiveMouseBtnCheck()
   return
}




SuspendToggle(Argument) {
   if (Argument = "On") {
      Suspend, On
   }
   else {
      Suspend, Off
   }
}


;-------- CONFIG ----------

Config_HotkeyGuiField() {
 global settingsDictionary
 ; This function runs when the gui-field for hotkey inputs are changed
 
 If %A_GuiControl% in +,^,!,+^,+!,^!,+^!    ;If the hotkey contains only modifiers, return to wait for a key.
  return
 
 name := SubStr(A_GuiControl,3)              ;Get the name of the hotkey control. Ignoring all letters before the 3rd letter, to ignore prefix
 
 If (HK%name% = "") {
   Gui, Submit, NoHide
 }
 Else {
   Gui, Submit, NoHide
   If CB%name%                                ;  If the 'Win' box is checked, then add its modifier (#).
      HK%name% := "#" HK%name%
   If !RegExMatch(HK%name%,"[#!\^\+]")        ;  If the new hotkey has no modifiers, add the (~) modifier.
      HK%name% := "~" HK%name%                   ;    This prevents any key from being blocked.
}
settingsDictionary["hotkeys", name] := HK%name%
return
}

Config_TextGuiField() {
 global settingsDictionary 
 Gui, Submit, NoHide
 inputValue := % %A_GuiControl%
 name := A_GuiControl
 settingsDictionary["settings", name] := inputValue
 return
}

Config_DisplayGuiField() {
 global DisplaySettings 
 Gui, Submit, nohide
 inputValue := % %A_GuiControl%
 name := A_GuiControl
 monNum := RegExReplace(name, "[^\d]")
 direction := SubStr(A_GuiControl,4)
 monitorID :=% "monitor"monNum
 DisplaySettings[monitorID, direction] := inputValue
 ;MsgBox, % DisplaySettings[displayName, direction]
 return
}



Config_CheckboxField() {
   global settingsDictionary
   Gui, Submit, nohide
   inputValue := % %A_GuiControl%
   name := A_GuiControl
   settingsDictionary["settings", name] := inputValue
}

WriteSettings() {
 global iniSettingFile
 global settingsDictionary
 global DisplaySettings
 
 ; --- WRITE HOTKEYS and SETTINGS
 for section, dict in settingsDictionary {
   for key, value in dict {
      IniWrite, %value%, %iniSettingFile%, %section%, %key%
   }
 }
 
 ; --- WRITE DISPLAY SETTINGS
for monitorID, dict in DisplaySettings {
 for direction, value in dict {
  IniWrite, %value%, %iniSettingFile%, %monitorID%, %direction%
 }
}
 MsgBox, Saved settings!`nRestart the application to update hotkeys.
 return
}

LoadSettings()

HK := settingsDictionary["hotkeys", "Default_ChangeMonitor"]
if HK {
   Hotkey, %HK%, Default_ChangeMonitor
}

HK := settingsDictionary["hotkeys", "Intuos_ChangeMonitor"]
if HK {
   Hotkey, %HK%, Intuos_ChangeMonitor
}

HK := settingsDictionary["hotkeys", "Cintiq_ChangeMonitor"]
if HK {
   Hotkey, %HK%, Cintiq_ChangeMonitor
}

HK := settingsDictionary["hotkeys", "Intuos_ToggleFullCoverage"]
if HK {
   Hotkey, %HK%, Intuos_ToggleFullCoverage
}

return

Default_ChangeMonitor:
ChangeTargetMonitor("default")
return

Intuos_ChangeMonitor:
ChangeTargetMonitor("intuos")
return

Cintiq_ChangeMonitor:
ChangeTargetMonitor("cintiq")
return

Intuos_ToggleFullCoverage:
ToggleFullCoverage()
return


Sus:
if (A_IsSuspended)
   SuspendToggle("Off")
Else
   SuspendToggle("On")
return

About:
MsgBox, 0, KS_WacomDisplayToggle, Developed by Kim Strandli`nwww.kimstrandli.com`n`nVersion: %version% `nReleased: %released%
return

Exi:
ExitApp
Return

Rel:
Reload
Return



;--------------------
;------ CONFIG LABELS/CMDs AND GUI-------------
;--------------------

GuiClose:
Gui Destroy
SuspendToggle("Off")
return


ConfirmConfig:
WriteSettings()
return


OpenConfigGui:
 SuspendToggle("On")

Gui, Font, s15
Gui, Add, Text, xm y+30, % "Hotkeys"  
Gui, Font
 
 hotkeyNames := ["Default_ChangeMonitor", "Cintiq_ChangeMonitor", "Intuos_ChangeMonitor", "Intuos_ToggleFullCoverage"]
 for index, name in hotkeyNames {
   Gui, Font,bold
  Gui, Add, Text, xm w160, %name%
   Gui, Font
  savedHK%name% := % settingsDictionary["hotkeys", name]
  StringReplace, noMods, savedHK%name%, ~                  ;Remove tilde (~) and Win (#) modifiers...
  StringReplace, noMods, noMods, #,,UseErrorLevel              ;They are incompatible with hotkey controls (cannot be shown).
  Gui, Add, Hotkey, x+5 vHK%name% gConfig_HotkeyGuiField, %noMods%           ;Add hotkey controls and show saved hotkeys. Input will be stored in global variable HK%name%
  Gui, Add, CheckBox, x+5 vCB%name% gConfig_HotkeyGuiField Checked%ErrorLevel%, % "+Windows Key"  ;Add checkboxes to allow the Windows key (#) as a modifier...
  if (InStr(savedHK%name%, "#")){
   guicontrol,, CB%name%, 1
  }
 }
;instructions := "WinKey must be enabled to combine with the windows-key."
;Gui, Add, Text, xm, %instructions%

Gui, Font, s15
Gui, Add, Text, xm y+30, % "Pens"  
Gui, Font

Gui, font,bold
Gui, Add, Text, xm, % "Cintiq Pen:"
Gui, font
Gui, Add, Edit, x+5 w60 vCintiqPenName gConfig_TextGuiField, 
penName := % settingsDictionary["settings", "CintiqPenName"]
guicontrol,, CintiqPenName, %penName%

Gui, font,bold
Gui, Add, Text, x+10, % "Intuos Pen:"
Gui, font
Gui, Add, Edit, x+5 w60 vIntuosPenName gConfig_TextGuiField, 
penName := % settingsDictionary["settings", "IntuosPenName"]
guicontrol,, IntuosPenName, %penName%

instructions := "Specify which pen you wish to modify if using more than one, otherwise first pen is used.`nExample: Pro Pen"
Gui, Add, Text, xm, %instructions%

Gui, Font, s15
Gui, Add, Text, xm y+30, % "Gesture Mapping"  
Gui, Font
 SysGet, MonCount, MonitorCount 
 Loop, %MonCount%
 {
   monNum := A_Index
   monitorID :=% "monitor"monNum
   Gui, font,bold
   Gui, Add, GroupBox, xm Section w350 h50, %monitorID%:
   Gui, font
   
   value := % DisplaySettings[monitorID, "force"]
   Gui, Add, Text, xs+5 ys+25, Force:
   Gui, Add, Edit, Limit1 Number x+5 ys+20 w20 vDT%monNum%force gConfig_DisplayGuiField, %value%
   
   value := % DisplaySettings[monitorID, "left"]
   Gui, Add, Text, x+15 ys+25, Left:
   Gui, Add, Edit, Limit1 Number x+5 ys+20 w20 vDT%monNum%left gConfig_DisplayGuiField, %value%
   
   value := % DisplaySettings[monitorID, "right"]
   Gui, Add, Text, x+15 ys+25, Right:
   Gui, Add, Edit, Limit1 Number x+5 ys+20 w20 vDT%monNum%right gConfig_DisplayGuiField, %value%
   
   value := % DisplaySettings[monitorID, "up"]
   Gui, Add, Text, x+15 ys+25, Up:
   Gui, Add, Edit, Limit1 Number x+5 ys+20 w20 vDT%monNum%up gConfig_DisplayGuiField, %value%
   
   value := % DisplaySettings[monitorID, "down"]
   Gui, Add, Text, x+15 ys+25, Down:
   Gui, Add, Edit, Limit1 Number x+5 ys+20 w20 vDT%monNum%down gConfig_DisplayGuiField, %value%
   
 }

Gui, Add, CheckBox, xm vDisableVerticalMovement gConfig_CheckboxField, Ignore vertical mouse movement
verticalvalue := % settingsDictionary["settings", "DisableVerticalMovement"]
if (verticalvalue = 1) {
   guicontrol,, DisableVerticalMovement, 1
}


 instructions := "Write the display-number you want to activate based on the pen gesture used on each display.`nWhen 'Force' is specified, all directional gestures are ignored.`nWhen only using two displays, the 'Force' option is recommended.`nUse Wacom Tablet Properties mapping and calibration menu to identify the display-numbers.`n"
 Gui, Add, Text, xm y+20, %instructions%

 instructions := "Hotkeys are disabled while the config window is open."
 Gui, Add, Text, xm y+20, %instructions%
 gui, add, button, xm gConfirmConfig, Confirm
 Gui, Show,,KSWacomDisplayToggle - Settings
 return
 






